@@ -52,6 +52,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA
 #ifndef GWENVIEW_SEMANTICINFO_BACKEND_NONE
 // KDE
 #include <nepomuk/kratingwidget.h>
+#ifdef GWENVIEW_SEMANTICINFO_BACKEND_NEPOMUK
+#include <nepomuk/resourcemanager.h>
+#endif
 
 // Local
 #include <lib/semanticinfo/abstractsemanticinfobackend.h>
@@ -533,8 +536,15 @@ FilterController::FilterController(QFrame* frame, SortedDirModel* dirModel)
 	d->addAction(i18nc("@action:inmenu", "Filter by Name"), SLOT(addFilterByName()));
 	d->addAction(i18nc("@action:inmenu", "Filter by Date"), SLOT(addFilterByDate()));
 #ifndef GWENVIEW_SEMANTICINFO_BACKEND_NONE
-	d->addAction(i18nc("@action:inmenu", "Filter by Rating"), SLOT(addFilterByRating()));
-	d->addAction(i18nc("@action:inmenu", "Filter by Tag"), SLOT(addFilterByTag()));
+#ifdef GWENVIEW_SEMANTICINFO_BACKEND_NEPOMUK
+	// Only add filters if Nepomuk is running
+	if (Nepomuk::ResourceManager::instance()->init() == 0) {
+#endif
+		d->addAction(i18nc("@action:inmenu", "Filter by Rating"), SLOT(addFilterByRating()));
+		d->addAction(i18nc("@action:inmenu", "Filter by Tag"), SLOT(addFilterByTag()));
+#ifdef GWENVIEW_SEMANTICINFO_BACKEND_NEPOMUK
+	}
+#endif
 #endif
 }
 
