@@ -1,6 +1,6 @@
 // vim: set tabstop=4 shiftwidth=4 noexpandtab:
 /*	Gwenview - A simple image viewer for KDE
-	Copyright 2000-2007 Aurélien Gâteau <aurelien.gateau@free.fr>
+	Copyright 2000-2007 Aurélien Gâteau <agateau@kde.org>
 	This class is based on the ImagePreviewJob class from Konqueror.
 */
 /*	This file is part of the KDE project
@@ -43,19 +43,11 @@
 #include <kdebug.h>
 #include <kde_file.h>
 #include <kfileitem.h>
-#include <kiconloader.h>
 #include <kio/jobuidelegate.h>
 #include <kio/previewjob.h>
 #include <kstandarddirs.h>
 #include <ktemporaryfile.h>
 
-// libjpeg
-#include <setjmp.h>
-#define XMD_H
-extern "C" {
-#include <jpeglib.h>
-}
-
 // Local
 #include "mimetypeutils.h"
 #include "jpegcontent.h"
@@ -88,6 +80,69 @@ static QString generateThumbnailPath(const QString& uri, ThumbnailGroup::Enum gr
 	return baseDir + QString(QFile::encodeName( md5.hexDigest())) + ".png";
 }
 
+//------------------------------------------------------------------------
+//
+// ThumbnailCache
+//
+//------------------------------------------------------------------------
+K_GLOBAL_STATIC(ThumbnailCache, sThumbnailCache)
+
+
+static void storeThumbnailToDiskCache(const QString& path, const QImage& image) {
+	LOG(path);
+	KTemporaryFile tmp;
+	tmp.setPrefix(path + ".gwenview.tmp");
+	tmp.setSuffix(".png");
+	if (!tmp.open()) {
+		kWarning() << "Could not create a temporary file.";
+		return;
+	}
+
+	if (!image.save(tmp.fileName(), "png")) {
+		kWarning() << "Could not save thumbnail";
+		return;
+	}
+
+	KDE_rename(QFile::encodeName(tmp.fileName()), QFile::encodeName(path));
+}
+
+
+void ThumbnailCache::queueThumbnail(const QString& path, const QImage& image) {
+	LOG(path);
+	QMutexLocker locker(&mMutex);
+	mCache.insert(path, image);
+}
+
+void ThumbnailCache::run() {
+	QMutexLocker locker(&mMutex);
+	while (true) {
+		Cache::ConstIterator it = mCache.constBegin();
+		if (it == mCache.constEnd()) {
+			break;
+		}
+		const QString path = it.key();
+		const QImage image = it.value();
+
+		// This part of the thread is the most time consuming but it does not
+		// depend on mCache so we can unlock here. This way other thumbnails
+		// can be added or queried
+		locker.unlock();
+		storeThumbnailToDiskCache(path, image);
+		locker.relock();
+
+		mCache.remove(path);
+		if (mCache.isEmpty()) {
+			break;
+		}
+	}
+}
+
+QImage ThumbnailCache::value(const QString& path) const {
+	QMutexLocker locker(&mMutex);
+	return mCache.value(path);
+}
+
+
 //------------------------------------------------------------------------
 //
 // ThumbnailThread
@@ -165,194 +220,83 @@ void ThumbnailThread::run() {
 
 void ThumbnailThread::loadThumbnail() {
 	mImage = QImage();
-	bool loaded=false;
 	bool needCaching=true;
 	int pixelSize = ThumbnailGroup::pixelSize(mThumbnailGroup);
+	Orientation orientation = NORMAL;
 
-	// If it's a Jpeg, try to load a small image directly from the file
-	if (isJpeg()) {
+	QImageReader reader(mPixPath);
+	// If it's a Jpeg, try to load an embedded thumbnail, if available
+	if (reader.format() == "jpeg") {
 		JpegContent content;
 		content.load(mPixPath);
 		mOriginalWidth = content.size().width();
 		mOriginalHeight = content.size().height();
-		mImage = content.thumbnail();
+		QImage thumbnail = content.thumbnail();
+		orientation = content.orientation();
 
-		if( !mImage.isNull()
-			&& ( mImage.width() >= pixelSize // don't use small thumbnails
-			|| mImage.height() >= pixelSize )) {
-			loaded = true;
+		if (qMax(thumbnail.width(), thumbnail.height()) >= pixelSize) {
+			mImage = thumbnail;
 			needCaching = false;
 		}
-#ifndef Q_OS_WIN32
-		if(!loaded) {
-			loaded = loadJpeg();
-		}
-#endif
-		if (loaded) {
-			// Rotate if necessary
-			Orientation orientation = content.orientation();
-			QMatrix matrix = ImageUtils::transformMatrix(orientation);
-			mImage = mImage.transformed(matrix);
-		}
 	}
 
-	// File is not a Jpeg, or Jpeg optimized load failed, load file using Qt
-	if (!loaded) {
-		QImage originalImage;
-		if (originalImage.load(mPixPath)) {
-			mOriginalWidth=originalImage.width();
-			mOriginalHeight=originalImage.height();
+	// Generate thumbnail from full image
+	if (mImage.isNull()) {
+		QSize originalSize = reader.size();
+		if (originalSize.isValid() && reader.supportsOption(QImageIOHandler::ScaledSize)) {
+			int scale;
+			const int maxSize = qMax(originalSize.width(), originalSize.height());
+			for (scale=1; pixelSize*scale*2 <= maxSize && scale <= 8; scale *= 2) {}
+			const QSize scaledSize = originalSize / scale;
+			if (!scaledSize.isEmpty()) {
+				reader.setScaledSize(scaledSize);
+			}
+		}
 
-			if (qMax(mOriginalWidth, mOriginalHeight)<=pixelSize ) {
+		QImage originalImage;
+		// format() is empty after QImageReader::read() is called
+		QByteArray format = reader.format();
+		if (reader.read(&originalImage)) {
+			if (!originalSize.isValid()) {
+				originalSize = originalImage.size();
+			}
+			mOriginalWidth = originalSize.width();
+			mOriginalHeight = originalSize.height();
+
+			if (qMax(mOriginalWidth, mOriginalHeight)<=pixelSize) {
 				mImage=originalImage;
-				needCaching = false;
+				needCaching = format != "png";
 			} else {
 				mImage = originalImage.scaled(pixelSize, pixelSize, Qt::KeepAspectRatio);
 			}
-			loaded = true;
 		}
 	}
 
+	if (mImage.isNull()) {
+		kWarning() << "Could not generate thumbnail for file" << mOriginalUri;
+		return;
+	}
+
+	// Rotate if necessary
+	if (orientation != NORMAL && orientation != NOT_AVAILABLE) {
+		QMatrix matrix = ImageUtils::transformMatrix(orientation);
+		mImage = mImage.transformed(matrix);
+	}
+
 	if (needCaching) {
-		mImage.setText("Thumb::Uri", 0, mOriginalUri);
-		mImage.setText("Thumb::MTime", 0, QString::number(mOriginalTime));
-		mImage.setText("Thumb::Size", 0, QString::number(mOriginalSize));
-		mImage.setText("Thumb::Mimetype", 0, mOriginalMimeType);
-		mImage.setText("Thumb::Image::Width", 0, QString::number(mOriginalWidth));
+		mImage.setText("Thumb::Uri"          , 0, mOriginalUri);
+		mImage.setText("Thumb::MTime"        , 0, QString::number(mOriginalTime));
+		mImage.setText("Thumb::Size"         , 0, QString::number(mOriginalSize));
+		mImage.setText("Thumb::Mimetype"     , 0, mOriginalMimeType);
+		mImage.setText("Thumb::Image::Width" , 0, QString::number(mOriginalWidth));
 		mImage.setText("Thumb::Image::Height", 0, QString::number(mOriginalHeight));
-		mImage.setText("Software", 0, "Gwenview");
+		mImage.setText("Software"            , 0, "Gwenview");
 
-		QString thumbnailDir = ThumbnailLoadJob::thumbnailBaseDir(mThumbnailGroup);
-		KStandardDirs::makeDir(thumbnailDir, 0700);
-
-		KTemporaryFile tmp;
-		tmp.setPrefix(thumbnailDir + "/gwenview");
-		tmp.setSuffix(".png");
-		if (!tmp.open()) {
-			kWarning() << "Could not create a temporary file.";
-			return;
-		}
-
-		if (!mImage.save(tmp.fileName(), "png")) {
-			kWarning() << "Could not save thumbnail for file" << mOriginalUri;
-			return;
-		}
-
-		KDE_rename(QFile::encodeName(tmp.fileName()), QFile::encodeName(mThumbnailPath));
+		emit thumbnailReadyToBeCached(mThumbnailPath, mImage);
 	}
 }
 
 
-bool ThumbnailThread::isJpeg() {
-	QString format = QImageReader::imageFormat(mPixPath);
-	return format == "jpeg";
-}
-
-
-
-struct JpegFatalError : public jpeg_error_mgr {
-	jmp_buf mJmpBuffer;
-
-	static void handler(j_common_ptr cinfo) {
-		JpegFatalError* error=static_cast<JpegFatalError*>(cinfo->err);
-		(error->output_message)(cinfo);
-		longjmp(error->mJmpBuffer,1);
-	}
-};
-
-bool ThumbnailThread::loadJpeg() {
-	struct jpeg_decompress_struct cinfo;
-
-	// Open file
-	FILE* inputFile = KDE_fopen(QFile::encodeName( mPixPath ).data(), "rb");
-	if(!inputFile) return false;
-
-	// Error handling
-	struct JpegFatalError jerr;
-	cinfo.err = jpeg_std_error(&jerr);
-	cinfo.err->error_exit = JpegFatalError::handler;
-	if (setjmp(jerr.mJmpBuffer)) {
-		jpeg_destroy_decompress(&cinfo);
-		fclose(inputFile);
-		return false;
-	}
-
-	// Init decompression
-	jpeg_create_decompress(&cinfo);
-	jpeg_stdio_src(&cinfo, inputFile);
-	jpeg_read_header(&cinfo, true);
-
-	// Get image size and check if we need a thumbnail
-	int size = ThumbnailGroup::pixelSize(mThumbnailGroup);
-	int imgSize = qMax(cinfo.image_width, cinfo.image_height);
-
-	if (imgSize<=size) {
-		fclose(inputFile);
-		return mImage.load(mPixPath);
-	}
-
-	// Compute scale value
-	int scale=1;
-	while(size*scale*2<=imgSize) {
-		scale*=2;
-	}
-	if(scale>8) scale=8;
-
-	cinfo.scale_num=1;
-	cinfo.scale_denom=scale;
-
-	// Create QImage
-	jpeg_start_decompress(&cinfo);
-
-	switch(cinfo.output_components) {
-	case 3:
-	case 4:
-		mImage = QImage(cinfo.output_width, cinfo.output_height, QImage::Format_RGB32);
-		break;
-	case 1: // B&W image
-		mImage = QImage(cinfo.output_width, cinfo.output_height, QImage::Format_Indexed8);
-		mImage.setNumColors(256);
-		for (int i=0; i<256; i++) {
-			mImage.setColor(i, qRgba(i, i, i, 255));
-		}
-		break;
-	default:
-		jpeg_destroy_decompress(&cinfo);
-		fclose(inputFile);
-		return false;
-	}
-
-	while (cinfo.output_scanline < cinfo.output_height) {
-		uchar *line = mImage.scanLine(cinfo.output_scanline);
-		jpeg_read_scanlines(&cinfo, &line, 1);
-	}
-	jpeg_finish_decompress(&cinfo);
-
-// Expand 24->32 bpp
-	if ( cinfo.output_components == 3 ) {
-		for (uint j=0; j<cinfo.output_height; j++) {
-			uchar *in = mImage.scanLine(j) + (cinfo.output_width - 1)*3;
-			QRgb *out = (QRgb*)( mImage.scanLine(j) ) + cinfo.output_width - 1;
-
-			for (int i=cinfo.output_width - 1; i>=0; --i, --out, in -= 3) {
-				*out = qRgb(in[0], in[1], in[2]);
-			}
-		}
-	}
-
-	int newMax = qMax(cinfo.output_width, cinfo.output_height);
-	int newx = size*cinfo.output_width / newMax;
-	int newy = size*cinfo.output_height / newMax;
-
-	mImage = mImage.scaled(newx, newy, Qt::KeepAspectRatio);
-
-	jpeg_destroy_decompress(&cinfo);
-	fclose(inputFile);
-
-	return true;
-}
-
-
 //------------------------------------------------------------------------
 //
 // ThumbnailLoadJob static methods
@@ -393,6 +337,26 @@ void ThumbnailLoadJob::deleteImageThumbnail(const KUrl& url) {
 }
 
 
+static void moveThumbnailHelper(const QString& oldUri, const QString& newUri, ThumbnailGroup::Enum group) {
+	QString oldPath = generateThumbnailPath(oldUri, group);
+	QString newPath = generateThumbnailPath(newUri, group);
+	QImage thumb;
+	if (!thumb.load(oldPath)) {
+		return;
+	}
+	thumb.setText("Thumb::Uri", 0, newUri);
+	thumb.save(newPath, "png");
+	QFile::remove(QFile::encodeName(oldPath));
+}
+
+void ThumbnailLoadJob::moveThumbnail(const KUrl& oldUrl, const KUrl& newUrl) {
+	QString oldUri = generateOriginalUri(oldUrl);
+	QString newUri = generateOriginalUri(newUrl);
+	moveThumbnailHelper(oldUri, newUri, ThumbnailGroup::Normal);
+	moveThumbnailHelper(oldUri, newUri, ThumbnailGroup::Large);
+}
+
+
 //------------------------------------------------------------------------
 //
 // ThumbnailLoadJob implementation
@@ -405,8 +369,9 @@ ThumbnailLoadJob::ThumbnailLoadJob(const KFileItemList& items, ThumbnailGroup::E
 {
 	LOG((int)this);
 
-	mBrokenPixmap = KIconLoader::global()->loadIcon("image-missing",
-		KIconLoader::NoGroup, 48);
+	// Make sure we have a place to store our thumbnails
+	QString thumbnailDir = ThumbnailLoadJob::thumbnailBaseDir(mThumbnailGroup);
+	KStandardDirs::makeDir(thumbnailDir, 0700);
 
 	// Look for images and store the items in our todo list
 	Q_ASSERT(!items.empty());
@@ -416,6 +381,10 @@ ThumbnailLoadJob::ThumbnailLoadJob(const KFileItemList& items, ThumbnailGroup::E
 	connect(&mThumbnailThread, SIGNAL(done(const QImage&, const QSize&)),
 		SLOT(thumbnailReady(const QImage&, const QSize&)),
 		Qt::QueuedConnection);
+
+	connect(&mThumbnailThread, SIGNAL(thumbnailReadyToBeCached(const QString&, const QImage&)),
+		sThumbnailCache, SLOT(queueThumbnail(const QString&, const QImage&)),
+		Qt::QueuedConnection);
 }
 
 
@@ -429,6 +398,9 @@ ThumbnailLoadJob::~ThumbnailLoadJob() {
 	}
 	mThumbnailThread.cancel();
 	mThumbnailThread.wait();
+	if (!sThumbnailCache->isRunning()) {
+		sThumbnailCache->start();
+	}
 }
 
 
@@ -589,12 +561,20 @@ void ThumbnailLoadJob::thumbnailReady( const QImage& _img, const QSize& _size) {
 	determineNextIcon();
 }
 
+QImage ThumbnailLoadJob::loadThumbnailFromCache() const {
+	QImage image = sThumbnailCache->value(mThumbnailPath);
+	if (!image.isNull()) {
+		return image;
+	}
+	return QImage(mThumbnailPath);
+}
+
 void ThumbnailLoadJob::checkThumbnail() {
 	// If we are in the thumbnail dir, just load the file
 	if (mCurrentUrl.isLocalFile()
 		&& mCurrentUrl.directory().startsWith(thumbnailBaseDir()) )
 	{
-		QImage image(mCurrentUrl.path());
+		QImage image(mCurrentUrl.toLocalFile());
 		emitThumbnailLoaded(image, image.size());
 		determineNextIcon();
 		return;
@@ -606,8 +586,8 @@ void ThumbnailLoadJob::checkThumbnail() {
 
 	LOG("Stat thumb" << mThumbnailPath);
 
-	QImage thumb;
-	if ( thumb.load(mThumbnailPath) ) {
+	QImage thumb = loadThumbnailFromCache();
+	if (!thumb.isNull()) {
 		if (thumb.text("Thumb::Uri", 0) == mOriginalUri &&
 			thumb.text("Thumb::MTime", 0).toInt() == mOriginalTime )
 		{
@@ -640,11 +620,10 @@ void ThumbnailLoadJob::checkThumbnail() {
 	}
 
 	// Thumbnail not found or not valid
-	if ( MimeTypeUtils::rasterImageMimeTypes().contains(mCurrentItem.mimetype()) ) {
-		// This is a raster image
+	if (MimeTypeUtils::fileItemKind(mCurrentItem) == MimeTypeUtils::KIND_RASTER_IMAGE) {
 		if (mCurrentUrl.isLocalFile()) {
 			// Original is a local file, create the thumbnail
-			startCreatingThumbnail(mCurrentUrl.path());
+			startCreatingThumbnail(mCurrentUrl.toLocalFile());
 		} else {
 			// Original is remote, download it
 			mState=STATE_DOWNLOADORIG;
@@ -705,8 +684,7 @@ void ThumbnailLoadJob::emitThumbnailLoaded(const QImage& img, const QSize& size)
 
 void ThumbnailLoadJob::emitThumbnailLoadingFailed() {
 	LOG(mCurrentItem.url());
-	QSize size;
-	emit thumbnailLoaded(mCurrentItem, mBrokenPixmap, size);
+	emit thumbnailLoadingFailed(mCurrentItem);
 }
 
 
