@@ -55,7 +55,9 @@
 #include <kio/job.h>
 #include <kicon.h>
 #include <kfilterdev.h>
+#if 0
 #include <knewstuff2/engine.h>
+#endif
 #include <kdeprintdialog.h>
 #include <kprintpreview.h>
 
@@ -610,6 +612,9 @@ m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args
     if ( m_embedMode != PrintPreviewMode )
         unsetDummyMode();
 
+    if ( m_embedMode == NativeShellMode )
+        m_sidebar->setAutoFillBackground( false );
+
 #ifdef OKULAR_KEEP_FILE_OPEN
     m_keeper = new FileKeeper();
 #endif
@@ -971,7 +976,7 @@ bool Part::openFile()
     if ( !ok )
     {
         // if can't open document, update windows so they display blank contents
-        m_pageView->widget()->update();
+        m_pageView->viewport()->update();
         m_thumbnailList->update();
         return false;
     }
@@ -1482,16 +1487,12 @@ void Part::slotSaveFileAs()
     if ( m_embedMode == PrintPreviewMode )
        return;
 
-    KUrl saveUrl = KFileDialog::getSaveUrl( KUrl("kfiledialog:///okular/" + url().fileName()), QString(), widget() );
+    KUrl saveUrl = KFileDialog::getSaveUrl( KUrl("kfiledialog:///okular/" + url().fileName()),
+                                            QString(), widget(), QString(),
+                                            KFileDialog::ConfirmOverwrite );
     if ( !saveUrl.isValid() || saveUrl.isEmpty() )
         return;
 
-    if ( KIO::NetAccess::exists( saveUrl, KIO::NetAccess::DestinationSide, widget() ) )
-    {
-        if (KMessageBox::warningContinueCancel( widget(), i18n("A file named \"%1\" already exists. Are you sure you want to overwrite it?", saveUrl.fileName()), QString(), KGuiItem(i18n("Overwrite"))) != KMessageBox::Continue)
-            return;
-    }
-
     KTemporaryFile tf;
     QString fileName;
     if ( !tf.open() )
@@ -1502,9 +1503,17 @@ void Part::slotSaveFileAs()
     fileName = tf.fileName();
     tf.close();
 
-    if ( !m_document->saveChanges( fileName ) )
+    QString errorText;
+    if ( !m_document->saveChanges( fileName, &errorText ) )
     {
-        KMessageBox::information( widget(), i18n("File could not be saved in '%1'. Try to save it to another location.", fileName ) );
+        if (errorText.isEmpty())
+        {
+            KMessageBox::information( widget(), i18n("File could not be saved in '%1'. Try to save it to another location.", fileName ) );
+        }
+        else
+        {
+            KMessageBox::information( widget(), i18n("File could not be saved in '%1'. %2", fileName, errorText ) );
+        }
         return;
     }
 
@@ -1519,15 +1528,11 @@ void Part::slotSaveCopyAs()
     if ( m_embedMode == PrintPreviewMode )
        return;
 
-    KUrl saveUrl = KFileDialog::getSaveUrl( KUrl("kfiledialog:///okular/" + url().fileName()), QString(), widget() );
+    KUrl saveUrl = KFileDialog::getSaveUrl( KUrl("kfiledialog:///okular/" + url().fileName()),
+                                            QString(), widget(), QString(),
+                                            KFileDialog::ConfirmOverwrite );
     if ( saveUrl.isValid() && !saveUrl.isEmpty() )
     {
-        if ( KIO::NetAccess::exists( saveUrl, KIO::NetAccess::DestinationSide, widget() ) )
-        {
-            if (KMessageBox::warningContinueCancel( widget(), i18n("A file named \"%1\" already exists. Are you sure you want to overwrite it?", saveUrl.fileName()), QString(), KGuiItem(i18n("Overwrite"))) != KMessageBox::Continue)
-                return;
-        }
-
         // make use of the already downloaded (in case of remote URLs) file,
         // no point in downloading that again
         KUrl srcUrl = KUrl::fromPath( localFilePath() );
@@ -1566,11 +1571,13 @@ void Part::slotSaveCopyAs()
 
 void Part::slotGetNewStuff()
 {
+#if 0
     KNS::Engine engine(widget());
     engine.init( "okular.knsrc" );
     // show the modal dialog over pageview and execute it
     KNS::Entry::List entries = engine.downloadDialogModal( m_pageView );
     Q_UNUSED( entries )
+#endif
 }
 
 
@@ -1618,6 +1625,10 @@ void Part::slotNewConfig()
     // update ThumbnailList contents
     if ( Okular::Settings::showLeftPanel() && !m_thumbnailList->isHidden() )
         m_thumbnailList->updateWidgets();
+
+    // update Reviews settings
+    if ( m_sidebar->isItemEnabled(2) )
+        m_reviewsWidget->reparseConfig();
 }
 
 
@@ -1639,6 +1650,10 @@ void Part::slotNewGeneratorConfig()
     // update ThumbnailList contents
     if ( Okular::Settings::showLeftPanel() && !m_thumbnailList->isHidden() )
         m_thumbnailList->updateWidgets();
+
+    // update Reviews settings
+    if ( m_sidebar->isItemEnabled(2) )
+        m_reviewsWidget->reparseConfig();
 }
 
 
@@ -1853,7 +1868,9 @@ void Part::slotExportAs(QAction * act)
             filter = m_exportFormats.at( id - 2 ).mimeType()->name();
             break;
     }
-    QString fileName = KFileDialog::getSaveFileName( url().isLocalFile() ? url().directory() : QString(), filter, widget() );
+    QString fileName = KFileDialog::getSaveFileName( url().isLocalFile() ? url().directory() : QString(),
+                                                     filter, widget(), QString(),
+                                                     KFileDialog::ConfirmOverwrite );
     if ( !fileName.isEmpty() )
     {
         bool saved = false;
